# -*- coding: utf-8 -*-
from decimal import Decimal
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_unicode


def base_label(report, field):
    if hasattr(field, 'verbose_name'):
        return "%s" % field.verbose_name.title()
    return field

base_lookup_label = lambda report, field: "[%s] %s" % (field.model._meta.verbose_name.title(), field.verbose_name.title())

model_lookup_label = lambda report, field: "[%s] %s" % (report.model._meta.verbose_name.title(), field.verbose_name.title())


def sum_column(values):
    if not values:
        return Decimal(0.00)
    if isinstance(values[0], (list, tuple)):
        return Decimal(sum([v if str.isdigit(str(v[0] if isinstance(v, (list, tuple)) else v)) else 1 for v in values]))
    return Decimal(sum(values))
sum_column.caption = _('Total')


def avg_column(values):
    if not values:
        return Decimal(0.00)
    return Decimal(float(sum_column(values)) / float(len(values)))
avg_column.caption = _('Average')


def count_column(values):
    return Decimal(len(values))
count_column.caption = _('Count')


def date_format(value, instance):
    return value.strftime("%d/%m/%Y")


def usd_format(value, instance):
    return 'USD %.2f' % Decimal(value)


def yesno_format(value, instance):
    return _('Yes') if value else _('No')


def round_format(value, instance):
    return Decimal('%.2f' % Decimal(value))


class ReportValue(object):
    value = None
    is_report_total = False
    is_group_total = False
    is_value = True

    def __init__(self, value):
        self.value = value

    def format(self, value, instance):
        return value

    def text(self):
        return force_unicode(self.format(self.value, instance=self))

    def __repr__(self):
        return self.text()

    def __unicode__(self):
        return self.text()

    def __str__(self):
        return "%s" % self.text()

    def __iter__(self):
        return self.value.__iter__()


class ReportRow(list):
    is_total = False
    is_caption = False

    def get_css_class(self):
        classes = []
        if self.is_total:
            classes.append('total')
        if self.is_caption:
            classes.append('caption')
        return " ".join(classes)

    def is_value(self):
        return self.is_total == False and self.is_caption == False



# -*- coding: utf-8 -*-
from django import forms
from django.template.loader import render_to_string
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode


class RangeWidget(forms.MultiWidget):
    def __init__(self, widget, *args, **kwargs):
        widgets = (widget, widget)
        kwargs['attrs'] = {'class': 'vDatepicker'}
        super(RangeWidget, self).__init__(widgets=widgets, *args, **kwargs)

    def decompress(self, value):
        raise value
        return value

    def format_output(self, rendered_widgets):
        widget_context = {'min': rendered_widgets[0], 'max': rendered_widgets[1]}
        return render_to_string('model_report/widgets/range_widget.html', widget_context)


class RangeField(forms.MultiValueField):
    default_error_messages = {
        'invalid_start': _(u'Enter a valid start value.'),
        'invalid_end': _(u'Enter a valid end value.'),
    }

    def __init__(self, field_class, widget=forms.TextInput, *args, **kwargs):
        if not 'initial' in kwargs:
            kwargs['initial'] = ['', '']

        fields = (field_class(), field_class())

        super(RangeField, self).__init__(
                fields=fields,
                widget=RangeWidget(widget),
                *args, **kwargs
                )
        self.label = force_unicode(field_class().label)

    def compress(self, data_list):
        if data_list:
            return [self.fields[0].clean(data_list[0]), self.fields[1].clean(data_list[1])]
        return None



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy
from django.utils.encoding import force_unicode


true = 'true'
false = 'false'
null = 'null'
undefined = 'undefined'
Solid = 'Solid'
outside = 'outside'


_ = lambda s: force_unicode(ugettext_lazy(s))


class CollectionObject(object):
    _dicts = null

    def __init__(self):
        self._dicts = []

    def add(self, obj):
        self._dicts.append(obj)

    def __repr__(self):
        return unicode(self._dicts)


class DictObject:

    def __init__(self, **default):
        x = dict([(k, v if isinstance(v, (DictObject, CollectionObject)) else null) for k, v in default.items()])
        self.__dict__.update(x)

    def update(self, **kwargs):
        self.__dict__.update(kwargs)

    def __repr__(self):
        data = {}
        for k, v in self.__dict__.items():
            if v != 'null':
                if isinstance(v, (type(ugettext_lazy(' ')))):
                    v = _(v)
                if isinstance(v, (bool)):
                    v = str(v).lower()
                if v == ('null',):
                    v = 'null'
                if unicode(v):
                    data[k] = v
        if not data:
            data = ""
        return unicode(data)

    def create(self, **defaults):
        obj = DictObject(**self.__dict__)
        obj.update(**defaults)
        return obj



# -*- coding: utf-8 -*-
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import Http404

from model_report.report import reports


def report_list(request):
    context = {
        'report_list': reports.get_reports()
    }
    return render_to_response('model_report/report_list.html', context, context_instance=RequestContext(request))


def report(request, slug):
    report = reports.get_report(slug)
    if not report:
        raise Http404
    context = {
        'report_list': reports.get_reports()
    }
    return report.render(request, extra_context=context)



# -*- coding: utf-8 -*-
from django import template
from django.template.loader import render_to_string


register = template.Library()


class ModelReportInlineNode(template.Node):
    def __init__(self, inline, row):
        self.inline = template.Variable(inline)
        self.row = template.Variable(row)

    def render(self, context):
        inline = self.inline.resolve(context)
        row = self.row.resolve(context)
        request = context.get('request')
        if row.is_value():
            inline_context = inline.get_render_context(request, by_row=row)
            if len(inline_context['report_rows']) > 0:
                return render_to_string('model_report/includes/report_inline.html', inline_context)
        return ''


@register.tag()
def model_report_render_inline(parser, token):
    try:
        tag_name, inline, row = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires arguments" % token.contents.split()[0])
    return ModelReportInlineNode(inline, row)



# -*- coding: utf-8 -*-
from django.utils.encoding import force_unicode
from django.utils.translation import ugettext_lazy as _
from model_report.highcharts.base import true, false, null, DictObject
from model_report.highcharts.options import get_highchart_data


def is_numeric(value):
    try:
        float(value)
    except (ValueError, TypeError):
        return False
    return True

from BeautifulSoup import BeautifulStoneSoup
import cgi


def HTMLEntitiesToUnicode(text):
    """Converts HTML entities to unicode.  For example '&amp;' becomes '&'."""
    text = unicode(BeautifulStoneSoup(text, convertEntities=BeautifulStoneSoup.ALL_ENTITIES))
    return text


def unicodeToHTMLEntities(text):
    """Converts unicode to HTML entities.  For example '&' becomes '&amp;'."""
    if text is None:
        text = force_unicode(_('None'))
    text = cgi.escape(text).encode('ascii', 'xmlcharrefreplace')
    return text


class HighchartRender(object):

    def reset(self):
        self.model = DictObject(**get_highchart_data())

    def __init__(self, config):
        self.reset()
        self.config = config

    def set_pie_chart_options(self, report_rows):
        funcs_op = {
            'sum': sum,
            'max': max,
            'min': min,
            'len': len,
            'avg': lambda vlist: sum(vlist) / len(vlist)
        }
        serie_operation = funcs_op[self.config['serie_op']]

        serie_data = []
        for grouper, rows in report_rows:
            add_group = True
            if self.config['has_report_totals']:
                if len(rows) <= 2:
                    add_group = False
            if not add_group:
                continue
            serie_values = []
            for row in rows:
                if row.is_value():
                    value = row[self.config['serie_field']].value
                    if not is_numeric(value):
                        value = 1  # TOOD: Map serie_field with posible serie_operator
                    serie_values.append(value)

            value = serie_operation(serie_values)
            grouper = unicodeToHTMLEntities(grouper)
            serie_data.append([grouper, round(value, 2)])
        data = self.model.serie_obj.create(**{
            'name': grouper,
            'data': serie_data,
            'type': 'pie',
        })
        self.model.series.add(data)

        self.model.chart.renderTo = 'container'
        self.model.chart.plotBackgroundColor = null,
        self.model.chart.plotBorderWidth = null,
        self.model.chart.plotShadow = false

        self.model.title.text = self.config['title']
        self.model.tooltip.formatter = "function() { return roundVal(this.percentage) + ' %'; }"

        self.model.plotOptions.pie.allowPointSelect = true
        self.model.plotOptions.pie.cursor = 'pointer'
        self.model.plotOptions.pie.dataLabels.enabled = true
        self.model.plotOptions.pie.dataLabels.color = '#000000'
        self.model.plotOptions.pie.dataLabels.connectorColor = '#000000'
        repr_char = '$'  # TODO: Fix this
        repr_fun = 'fm'
        if self.config['serie_op'] == 'len':
            repr_char = ''
            repr_fun = ''
        self.model.plotOptions.pie.dataLabels.formatter = "function() { return '<b>'+ this.point.name +'</b>: %s '+ %s(this.point.y); }" % (repr_char, repr_fun)

    def set_bar_chart_options(self, report_rows):
        funcs_op = {
            'sum': sum,
            'max': max,
            'min': min,
            'len': len,
            'avg': lambda vlist: sum(vlist) / len(vlist)
        }
        serie_operation = funcs_op[self.config['serie_op']]

        serie_data = []
        xAxis_categories = []
        yAxis_min = 0
        for grouper, rows in report_rows:
            add_group = True
            if self.config['has_report_totals']:
                if len(rows) <= 2:
                    add_group = False
            if not add_group:
                continue
            serie_values = []
            for r in rows:
                if r.is_value():
                    value = r[self.config['serie_field']].value
                    if not is_numeric(value):
                        value = 1  # TOOD: Map serie_field with posible serie_operator
                    serie_values.append(value)

            value = serie_operation(serie_values)
            grouper = unicodeToHTMLEntities(grouper)
            serie_data.append(round(value, 2))
            xAxis_categories.append(grouper)
            yAxis_min = yAxis_min if value > yAxis_min else value
        data = self.model.serie_obj.create(**{
            'name': grouper,
            'data': serie_data,
        })
        self.model.series.add(data)

        self.model.chart.renderTo = 'container'
        self.model.chart.type = 'column'
        self.model.title.text = self.config['title']
        self.model.xAxis.categories = xAxis_categories
        self.model.xAxis.min = yAxis_min
        self.model.yAxis.title.text = ' '
        self.model.tooltip.formatter = "function() { return ''+ this.x +': '+ this.y; }"
        self.model.plotOptions.column.pointPadding = 0.2
        self.model.plotOptions.column.borderWidth = 0.0
        self.model.plotOptions.column.colorByPoint = true
        self.model.legend.enabled = false

    def set_line_chart_options(self, report_rows):
        xAxis_categories = []
        max_length = 0
        for grouper, rows in report_rows:
            add_group = True
            if self.config['has_report_totals']:
                if len(rows) <= 2:
                    add_group = False
            if not add_group:
                continue
            serie_values = []
            for row in rows:
                if row.is_value():
                    value = row[self.config['serie_field']].value
                    if not is_numeric(value):
                        value = 1  # TOOD: Map serie_field with posible serie_operator
                    serie_values.append(float(value))

            grouper = unicodeToHTMLEntities(grouper)
            xAxis_categories.append(grouper)
            data = self.model.serie_obj.create(**{
                'name': grouper,
                'data': serie_values,
            })
            self.model.series.add(data)
            if len(serie_values) > max_length:
                max_length = len(serie_values)

        xAxis_categories = range(1, max_length + 1)
        self.model.chart.renderTo = 'container'
        self.model.chart.type = 'line'

        self.model.title.text = self.config['title']
        self.model.xAxis.categories = xAxis_categories

        self.model.yAxis.title.text = 'USD'

        self.model.plotOptions.line.dataLabels.edabled = true
        self.model.plotOptions.line.enableMouseTracking = false

        self.model.tooltip.enabled = true
        self.model.tooltip.formatter = "function() { return '<b>'+this.series.name+'</b> '+this.x+': '+this.y; }"

    def is_valid(self):
        if self.config:
            if 'serie_field' in self.config and not self.config['serie_field'] is None:
                return True
        return False

    def get_chart(self, report_rows):
        self.reset()
        if report_rows and self.is_valid():
            if self.config['chart_mode'] == 'pie':
                self.model.credits.enabled = false
                self.set_pie_chart_options(report_rows)
            if self.config['chart_mode'] == 'column':
                self.model.credits.enabled = false
                self.set_bar_chart_options(report_rows)
            if self.config['chart_mode'] == 'line':
                self.model.credits.enabled = false
                self.set_line_chart_options(report_rows)
        return self

    @property
    def options(self):
        from django.utils import simplejson
        json = unicode(self.model)
        json = simplejson.dumps(json)[1:-1]
        json = json.replace("'true'", 'true')
        json = json.replace("'false'", 'false')
        json = json.replace("'null'", 'null')
        json = json.replace('\\"', '')
        json = json.replace('},', '},\n\t')
        json = json.replace('[{', '[\n\t{')
        json = json.replace('}]', '}\n]')
        json = json.replace("u'", "'")
        json = HTMLEntitiesToUnicode(json)
        return json



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *
from model_report.views import report, report_list

urlpatterns = patterns('',
    url(r'^$', report_list, name='model_report_list'),

    url(r'^(?P<slug>[\w-]+)/$', report, name='model_report_view'),
)



# -*- coding: utf-8 -*-
import copy
import csv
from itertools import groupby

from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.db.models.fields import DateTimeField, DateField
from django.utils.encoding import force_unicode
from django.db.models import Q
from django import forms
from django.forms.models import fields_for_model
from django.db.models.related import RelatedObject
from django.conf import settings


from model_report.utils import base_label, ReportValue, ReportRow
from model_report.highcharts import HighchartRender
from model_report.widgets import RangeField
from model_report.export_pdf import render_to_pdf
from django.utils.datetime_safe import datetime
import time


try:
    from collections import OrderedDict
except:
    OrderedDict = dict  # pyflakes:ignore


def autodiscover():
    """
    Auto-discover INSTALLED_APPS report.py modules and fail silently when
    not present. Borrowed form django.contrib.admin
    """

    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    global reports

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's admin module.
        try:
            before_import_registry = copy.copy(reports)
            import_module('%s.reports' % app)
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            # (see #8245).
            reports = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an admin module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'reports'):
                raise


class ReportInstanceManager(object):

    _register = OrderedDict()

    def __init__(self):
        self._register = OrderedDict()

    def register(self, slug, rclass):
        if slug in self._register:
            raise ValueError('Slug already exists: %s' % slug)
        report = rclass()
        setattr(report, 'slug', slug)
        self._register[slug] = report

    def get_report(self, slug):
        return self._register.get(slug, None)

    def get_reports(self):
        return self._register.values()


reports = ReportInstanceManager()


_cache_class = {}


def cache_return(fun):
    def wrap(self, *args, **kwargs):
        cache_field = '%s_%s' % (self.__class__.__name__, fun.func_name)
        if cache_field in _cache_class:
            return _cache_class[cache_field]
        result = fun(self, *args, **kwargs)
        _cache_class[cache_field] = result
        return result
    return wrap


class ReportAdmin(object):

    fields = []
    model = None
    list_filter = ()
    list_order_by = ()
    list_group_by = ()
    list_serie_fields = ()
    template_name = None
    title = None
    type = 'report'
    group_totals = {}
    report_totals = {}
    override_field_values = {}
    override_field_formats = {}
    override_field_labels = {}
    override_field_choices = {}
    override_field_filter_values = {}
    override_group_value = {}
    chart_types = ()
    exports = ('excel', 'pdf')
    inlines = []
    query_set = None

    def __init__(self, parent_report=None, request=None):
        self.parent_report = parent_report
        self.request = request
        model_fields = []
        model_m2m_fields = []
        self.related_inline_field = None
        self.related_inline_accessor = None
        self.related_fields = []
        if parent_report:
            self.related_inline_field = [f for f, x in self.model._meta.get_fields_with_model() if f.rel and hasattr(f.rel, 'to') and f.rel.to is self.parent_report.model][0]
            self.related_inline_accessor = self.related_inline_field.related.get_accessor_name()
            self.related_fields = ["%s__%s" % (pfield.model._meta.module_name, attname) for pfield, attname in self.parent_report.model_fields if pfield.model == self.related_inline_field.rel.to]
        for field in self.get_query_field_names():
            try:
                m2mfields = []
                if '__' in field:  # IF field has lookup
                    pre_field = None
                    base_model = self.model
                    for field_lookup in field.split("__"):
                        if not pre_field:
                            pre_field = base_model._meta.get_field_by_name(field_lookup)[0]
                            if 'ManyToManyField' in unicode(pre_field) or isinstance(pre_field, RelatedObject):
                                m2mfields.append(pre_field)
                        elif isinstance(pre_field, RelatedObject):
                            base_model = pre_field.model
                            pre_field = base_model._meta.get_field_by_name(field_lookup)[0]
                        else:
                            if 'Date' in unicode(pre_field):
                                pre_field = pre_field
                            else:
                                base_model = pre_field.rel.to
                                pre_field = base_model._meta.get_field_by_name(field_lookup)[0]
                    model_field = pre_field
                else:
                    if not 'self.' in field:
                        model_field = self.model._meta.get_field_by_name(field)[0]
                    else:
                        get_attr = lambda s: getattr(s, field.split(".")[1])
                        get_attr.verbose_name = field
                        model_field = field
            except IndexError:
                raise ValueError('The field "%s" does not exist in model "%s".' % (field, self.model._meta.module_name))
            model_fields.append([model_field, field])
            if m2mfields:
                model_m2m_fields.append([model_field, field, len(model_fields) - 1, m2mfields])
        self.model_fields = model_fields
        self.model_m2m_fields = model_m2m_fields

    def _get_grouper_text(self, groupby_field, value):
        try:
            model_field = [mfield for mfield, field in self.model_fields if field == groupby_field][0]
        except:
            model_field = None
        value = self.get_grouper_text(value, groupby_field, model_field)
        if value is None or unicode(value) == u'None':
            if groupby_field is None or unicode(groupby_field) == u'None':
                value = force_unicode(_('Results'))
            else:
                value = force_unicode(_('Nothing'))
        return value

    def _get_value_text(self, index, value):
        try:
            model_field = self.model_fields[index][0]
        except:
            model_field = None

        value = self.get_value_text(value, index, model_field)
        if value is None or unicode(value) == u'None':
            value = ''
        return value

    def get_grouper_text(self, value, field, model_field):
        try:
            if not isinstance(model_field, (str, unicode)):
                obj = model_field.model(**{field: value})
                if hasattr(obj, 'get_%s_display' % field):
                    value = getattr(obj, 'get_%s_display' % field)()
        except:
            pass
        return value

    @cache_return
    def get_m2m_field_names(self):
        return [field for ffield, field, index, mfield in self.model_m2m_fields]

    def get_value_text(self, value, index, model_field):
        try:
            if not isinstance(model_field, (str, unicode)):
                obj = model_field.model(**{model_field.name: value})
                if hasattr(obj, 'get_%s_display' % model_field.name):
                    return getattr(obj, 'get_%s_display' % model_field.name)()
        except:
            pass
        return value

    def get_empty_row_asdict(self, collection, default_value=[]):
        erow = {}
        for field in collection:
            erow[field] = copy.copy(default_value)
        return dict(copy.deepcopy(erow))

    def reorder_dictrow(self, dictrow):
        return [dictrow[field_name] for field_name in self.get_fields()]

    def get_fields(self):
        return [x for x in self.fields if not x in self.related_fields]

    def get_column_names(self, ignore_columns={}):
        values = []
        for field, field_name in self.model_fields:
            if field_name in ignore_columns:
                continue
            caption = self.override_field_labels.get(field_name, base_label)(self, field)
            values.append(caption)
        return values

    @cache_return
    def get_query_field_names(self):
        values = []
        for field in self.get_fields():
            if not 'self.' in field:
                values.append(field.split(".")[0])
            else:
                values.append(field)
        return values

    @cache_return
    def get_query_set(self, filter_kwargs):
        qs = self.model.objects.all()
        for k, v in filter_kwargs.items():
            if not v is None and v != '':
                if hasattr(v, 'values_list'):
                    v = v.values_list('pk', flat=True)
                    k = '%s__pk__in' % k.split("__")[0]
                qs = qs.filter(Q(**{k: v}))
        self.query_set = qs.distinct()
        return self.query_set

    def get_title(self):
        title = self.title or None
        if not title:
            if not self.model:
                title = _('Unnamed report')
            else:
                title = force_unicode(self.model._meta.verbose_name_plural).lower().capitalize()
        return title

    def get_render_context(self, request, extra_context={}, by_row=None):
        context_request = request or self.request
        related_fields = []
        filter_related_fields = {}
        if self.parent_report and by_row:

            for index, (pfield, plookup) in enumerate(self.parent_report.model_fields):
                for cfield, clookup in self.model_fields:
                    if pfield is cfield and plookup in clookup:
                        related_fields.append([pfield, cfield, plookup, clookup, by_row[index].value])
                        filter_related_fields[clookup] = by_row[index].value

        try:
            form_groupby = self.get_form_groupby(context_request)
            form_filter = self.get_form_filter(context_request)
            form_config = self.get_form_config(context_request)

            column_labels = self.get_column_names(filter_related_fields)
            report_rows = []
            groupby_data = None
            filter_kwargs = None
            report_anchors = []
            chart = None

            context = {
                'report': self,
                'form_groupby': form_groupby,
                'form_filter': form_filter,
                'form_config': form_config if self.type == 'chart' else None,
                'chart': chart,
                'report_anchors': report_anchors,
                'column_labels': column_labels,
                'report_rows': report_rows,
            }

            if context_request.GET:
                groupby_data = form_groupby.get_cleaned_data() if form_groupby else None
                filter_kwargs = filter_related_fields or form_filter.get_filter_kwargs()
                if groupby_data:
                    self.__dict__.update(groupby_data)
                else:
                    self.__dict__['onlytotals'] = False
                report_rows = self.get_rows(context_request, groupby_data, filter_kwargs, filter_related_fields)

                for g, r in report_rows:
                    report_anchors.append(g)

                if len(report_anchors) <= 1:
                    report_anchors = []

                if self.type == 'chart' and groupby_data and groupby_data['groupby']:
                    config = form_config.get_config_data()
                    if config:
                        chart = self.get_chart(config, report_rows)

                if self.onlytotals:
                    for g, rows in report_rows:
                        for r in list(rows):
                            if r.is_value():
                                rows.remove(r)

                if not context_request.GET.get('export', None) is None and not self.parent_report:
                    if context_request.GET.get('export') == 'excel':
                        response = HttpResponse(mimetype='text/csv')
                        response['Content-Disposition'] = 'attachment; filename=%s.csv' % self.slug

                        writer = csv.writer(response)

                        writer.writerow([unicode(x).encode("utf-8") for x in column_labels])

                        for g, rows in report_rows:
                            for row in list(rows):
                                if row.is_value():
                                    writer.writerow([unicode(x.value).encode("utf-8") for x in row])
                                elif row.is_caption:
                                    writer.writerow([unicode(x).encode("utf-8") for x in row])
                                elif row.is_total:
                                    writer.writerow([unicode(x.value).encode("utf-8") for x in row])
                                    writer.writerow([unicode(' ').encode("utf-8") for x in row])

                        return response
                    if context_request.GET.get('export') == 'pdf':
                        inlines = [ir(self, context_request) for ir in self.inlines]
                        report_anchors = None
                        setattr(self, 'is_export', True)
                        context = {
                            'report': self,
                            'column_labels': column_labels,
                            'report_rows': report_rows,
                            'report_inlines': inlines,
                        }
                        context.update({'pagesize': 'legal landscape'})
                        return render_to_pdf(self, 'model_report/export_pdf.html', context)

            inlines = [ir(self, context_request) for ir in self.inlines]

            is_inline = self.parent_report is None
            render_report = not (len(report_rows) == 0 and is_inline)
            context = {
                'render_report': render_report,
                'is_inline': is_inline,
                'inline_column_span': 0 if is_inline else len(self.parent_report.get_column_names()),
                'report': self,
                'form_groupby': form_groupby,
                'form_filter': form_filter,
                'form_config': form_config if self.type == 'chart' else None,
                'chart': chart,
                'report_anchors': report_anchors,
                'column_labels': column_labels,
                'report_rows': report_rows,
                'report_inlines': inlines,
            }

            if extra_context:
                context.update(extra_context)

            context['request'] = request
            return context
        finally:
            globals()['_cache_class'] = {}

    def render(self, request, extra_context={}):
        context_or_response = self.get_render_context(request, extra_context)

        if isinstance(context_or_response, HttpResponse):
            return context_or_response
        return render_to_response('model_report/report.html', context_or_response, context_instance=RequestContext(request))

    def has_report_totals(self):
        return not (not self.report_totals)

    def has_group_totals(self):
        return not (not self.group_totals)

    def get_chart(self, config, report_rows):
        config['title'] = self.get_title()
        config['has_report_totals'] = self.has_report_totals()
        config['has_group_totals'] = self.has_group_totals()
        return HighchartRender(config).get_chart(report_rows)

    def get_form_config(self, request):

        DEFAULT_CHART_TYPES = (
            ('area', _('Area')),
            ('line', _('Line')),
            ('column', _('Columns')),
            ('pie', _('Pie')),
        )
        CHART_SERIE_OPERATOR = (
            ('', '---------'),
            ('sum', _('Sum')),
            ('len', _('Count')),
            ('avg', _('Average')),
            ('min', _('Min')),
            ('max', _('Max')),
        )

        class ConfigForm(forms.Form):

            chart_mode = forms.ChoiceField(label=_('Chart type'), choices=(), required=False)
            serie_field = forms.ChoiceField(label=_('Serie field'), choices=(), required=False)
            serie_op = forms.ChoiceField(label=_('Serie operator'), choices=CHART_SERIE_OPERATOR, required=False)

            def __init__(self, *args, **kwargs):
                super(ConfigForm, self).__init__(*args, **kwargs)
                choices = [('', '')]
                for k, v in DEFAULT_CHART_TYPES:
                    if k in self.chart_types:
                        choices.append([k, v])
                self.fields['chart_mode'].choices = list(choices)
                choices = [('', '')]
                for i, (index, mfield, field, caption) in enumerate(self.serie_fields):
                    choices += (
                        (index, caption),
                    )
                self.fields['serie_field'].choices = list(choices)

            def get_config_data(self):
                data = getattr(self, 'cleaned_data', {})
                if not data:
                    return {}
                if not data['serie_field'] or not data['chart_mode'] or not data['serie_op']:
                    return {}
                data['serie_field'] = int(data['serie_field'])
                return data

        ConfigForm.serie_fields = self.get_serie_fields()
        ConfigForm.chart_types = self.chart_types
        ConfigForm.serie_fields
        form = ConfigForm(data=request.GET or None)
        form.is_valid()

        return form

    @cache_return
    def get_groupby_fields(self):
        return [(mfield, field, caption) for (mfield, field), caption in zip(self.model_fields, self.get_column_names()) if field in self.list_group_by]

    @cache_return
    def get_serie_fields(self):
        return [(index, mfield, field, caption) for index, ((mfield, field), caption) in enumerate(zip(self.model_fields, self.get_column_names())) if field in self.list_serie_fields]

    @cache_return
    def get_form_groupby(self, request):
        groupby_fields = self.get_groupby_fields()

        if not groupby_fields:
            return None

        class GroupByForm(forms.Form):

            groupby = forms.ChoiceField(label=_('Group by field:'), required=False)
            onlytotals = forms.BooleanField(label=_('Show only totals'), required=False)

            def _post_clean(self):
                pass

            def __init__(self, **kwargs):
                super(GroupByForm, self).__init__(**kwargs)
                choices = [(None, '')]
                for i, (mfield, field, caption) in enumerate(self.groupby_fields):
                    choices.append((field, caption))
                self.fields['groupby'].choices = choices
                data = kwargs.get('data', {})
                if data:
                    self.fields['groupby'].initial = data.get('groupby', '')

            def get_cleaned_data(self):
                cleaned_data = getattr(self, 'cleaned_data', {})
                if 'groupby' in cleaned_data:
                    if unicode(cleaned_data['groupby']) == u'None':
                        cleaned_data['groupby'] = None
                return cleaned_data

        GroupByForm.groupby_fields = groupby_fields

        form = GroupByForm(data=request.GET or None)
        form.is_valid()

        return form

    def get_form_filter(self, request):
        form_fields = fields_for_model(self.model, [f for f in self.get_query_field_names() if f in self.list_filter])
        if not form_fields:
            form_fields = {
                '__all__': forms.BooleanField(label='', widget=forms.HiddenInput, initial='1')
            }
        else:
            opts = self.model._meta
            for k, v in dict(form_fields).items():
                if v is None:
                    pre_field = None
                    base_model = self.model
                    if '__' in k:
                        for field_lookup in k.split("__")[:-1]:
                            if pre_field:
                                if isinstance(pre_field, RelatedObject):
                                    base_model = pre_field.model
                                else:
                                    base_model = pre_field.rel.to
                            pre_field = base_model._meta.get_field_by_name(field_lookup)[0]

                        model_field = pre_field
                    else:
                        field_name = k.split("__")[0]
                        model_field = opts.get_field_by_name(field_name)[0]

                    if isinstance(model_field, (DateField, DateTimeField)):
                        form_fields.pop(k)
                        field = RangeField(model_field.formfield)
                    else:
                        if not hasattr(model_field, 'formfield'):
                            field = forms.ModelChoiceField(queryset=model_field.model.objects.all())
                            field.label = self.override_field_labels.get(k, base_label)(self, field) if k in self.override_field_labels else field_lookup
                        else:
                            field = model_field.formfield()
                        field.label = force_unicode(_(field.label))

                else:
                    if isinstance(v, (forms.BooleanField)):
                        form_fields.pop(k)
                        field = forms.ChoiceField()
                        field.label = v.label
                        field.help_text = v.help_text
                        field.choices = (
                            ('', ''),
                            (True, _('Yes')),
                            (False, _('No')),
                        )
                        setattr(field, 'as_boolean', True)
                    elif isinstance(v, (forms.DateField, forms.DateTimeField)):
                        field_name = k.split("__")[0]
                        model_field = opts.get_field_by_name(field_name)[0]
                        form_fields.pop(k)
                        field = RangeField(model_field.formfield)
                    else:
                        field = v

                    if hasattr(field, 'choices'):
                        # self.override_field_filter_values
                        if not hasattr(field, 'queryset'):
                            if field.choices[0][0]:
                                field.choices.insert(0, ('', '---------'))
                                field.initial = ''

                # Provide a hook for updating the queryset
                if hasattr(field, 'queryset') and k in self.override_field_choices:
                    field.queryset = self.override_field_choices.get(k)(self, field.queryset)
                form_fields[k] = field

        form_class = type('FilterFormBase', (forms.BaseForm,), {'base_fields': form_fields})

        class FilterForm(form_class):

            def _post_clean(self):
                pass

            def get_filter_kwargs(self):
                if not self.is_valid():
                    return {}
                filter_kwargs = dict(self.cleaned_data)
                for k, v in dict(filter_kwargs).items():
                    if not v:
                        filter_kwargs.pop(k)
                        continue
                    if k == '__all__':
                        filter_kwargs.pop(k)
                        continue
                    if isinstance(v, (list, tuple)):
                        if isinstance(self.fields[k], (RangeField)):
                            filter_kwargs.pop(k)
                            start_range, end_range = v
                            if start_range:
                                filter_kwargs['%s__gte' % k] = start_range
                            if end_range:
                                filter_kwargs['%s__lte' % k] = end_range
                    elif hasattr(self.fields[k], 'as_boolean'):
                        if v:
                            filter_kwargs.pop(k)
                            filter_kwargs[k] = (unicode(v) == u'True')
                return filter_kwargs

            def get_cleaned_data(self):
                return getattr(self, 'cleaned_data', {})

            def __init__(self, *args, **kwargs):
                super(FilterForm, self).__init__(*args, **kwargs)
                self.filter_report_is_all = '__all__' in self.fields and len(self.fields) == 1
                try:
                    data_filters = {}
                    vals = args[0]
                    for k in vals.keys():
                        if k in self.fields:
                            data_filters[k] = vals[k]
                    for name in self.fields:
                        for k, v in data_filters.items():
                            if k == name:
                                continue
                            field = self.fields[name]
                            if hasattr(field, 'queryset'):
                                qs = field.queryset
                                if k in qs.model._meta.get_all_field_names():
                                    field.queryset = qs.filter(Q(**{k: v}))
                except:
                    pass

                for field in self.fields:
                    self.fields[field].required = False

        form = FilterForm(data=request.GET or None)
        form.is_valid()

        return form

    def filter_query(self, qs):
        return qs

    def get_rows(self, request, groupby_data=None, filter_kwargs={}, filter_related_fields={}):
        report_rows = []

        def get_field_value(obj, field):
            if isinstance(obj, (dict)):
                return obj[field]
            left_field = field.split("__")[0]
            try:
                right_field = "__".join(field.split("__")[1:])
            except:
                right_field = ''
            if right_field:
                return get_field_value(getattr(obj, left_field), right_field)
            if hasattr(obj, 'get_%s_display' % left_field):
                attr = getattr(obj, 'get_%s_display' % field)
            else:
                attr = getattr(obj, field)
            if callable(attr):
                attr = attr()
            return attr

        for kwarg, value in filter_kwargs.items():
            if kwarg in self.override_field_filter_values:
                filter_kwargs[kwarg] = self.override_field_filter_values.get(kwarg)(self, value)

        qs = self.get_query_set(filter_kwargs)
        ffields = [f if 'self.' not in f else 'pk' for f in self.get_query_field_names() if f not in filter_related_fields]
        extra_ffield = []
        backend = settings.DATABASES['default']['ENGINE'].split('.')[-1]
        for f in list(ffields):
            if '__' in f:
                for field, name in self.model_fields:
                    if name == f:
                        if 'fields.Date' in unicode(field):
                            fname, flookup = f.split('__')
                            if not flookup in ('year', 'month', 'day'):
                                break
                            if flookup == 'year':
                                if 'sqlite' in backend:
                                    extra_ffield.append([f, "strftime('%%Y', " + fname + ")"])
                                elif 'postgres' in backend:
                                    extra_ffield.append([f, "cast(extract(year from " + fname + ") as integer)"])
                                else:
                                    raise NotImplemented  # mysql
                            if flookup == 'month':
                                if 'sqlite' in backend:
                                    extra_ffield.append([f, "strftime('%%m', " + fname + ")"])
                                elif 'postgres' in backend:
                                    extra_ffield.append([f, "cast(extract(month from " + fname + ") as integer)"])
                                else:
                                    raise NotImplemented  # mysql
                            if flookup == 'day':
                                if 'sqlite' in backend:
                                    extra_ffield.append([f, "strftime('%%d', " + fname + ")"])
                                elif 'postgres' in backend:
                                    extra_ffield.append([f, "cast(extract(day from " + fname + ") as integer)"])
                                else:
                                    raise NotImplemented  # mysql
                        break
        obfields = list(self.list_order_by)
        if groupby_data and groupby_data['groupby']:
            if groupby_data['groupby'] in obfields:
                obfields.remove(groupby_data['groupby'])
            obfields.insert(0, groupby_data['groupby'])
        qs = self.filter_query(qs)
        qs = qs.order_by(*obfields)
        if extra_ffield:
            qs = qs.extra(select=dict(extra_ffield))
        qs = qs.values_list(*ffields)
        qs_list = list(qs)

        def get_with_dotvalues(resources):
            # {1: 'field.method'}
            dot_indexes = dict([(index, dot_field) for index, dot_field in enumerate(self.get_fields()) if '.' in dot_field])
            dot_indexes_values = {}

            dot_model_fields = [(index, model_field[0]) for index, model_field in enumerate(self.model_fields) if index in dot_indexes]
            # [ 1, model_field] ]
            for index, model_field in dot_model_fields:
                model_ids = set([row[index] for row in resources])
                if isinstance(model_field, (unicode, str)) and 'self.' in model_field:
                    model_qs = self.model.objects.filter(pk__in=model_ids)
                else:
                    model_qs = model_field.rel.to.objects.filter(pk__in=model_ids)
                div = {}
                method_name = dot_indexes[index].split('.')[1]
                for obj in model_qs:
                    method_value = getattr(obj, method_name)
                    if callable(method_value):
                        method_value = method_value()
                    div[obj.pk] = method_value
                dot_indexes_values[index] = div
                del model_qs

            if dot_indexes_values:
                new_resources = []
                for index_row, old_row in enumerate(resources):
                    new_row = []
                    for index, actual_value in enumerate(old_row):
                        if index in dot_indexes_values:
                            new_value = dot_indexes_values[index][actual_value]
                        else:
                            new_value = actual_value
                        new_row.append(new_value)
                    new_resources.append(new_row)
                resources = new_resources
            return resources

        def compute_row_totals(row_config, row_values, is_group_total=False, is_report_total=False):
            total_row = self.get_empty_row_asdict(self.get_fields(), ReportValue(' '))
            for k, v in total_row.items():
                if k in row_config:
                    fun = row_config[k]
                    value = fun(row_values[k])
                    if k in self.get_m2m_field_names():
                        value = ReportValue([value, ])
                    value = ReportValue(value)
                    value.is_value = False
                    value.is_group_total = is_group_total
                    value.is_report_total = is_report_total
                    if k in self.override_field_values:
                        value.to_value = self.override_field_values[k]
                    if k in self.override_field_formats:
                        value.format = self.override_field_formats[k]
                    value.is_m2m_value = (k in self.get_m2m_field_names())
                    total_row[k] = value
            row = self.reorder_dictrow(total_row)
            row = ReportRow(row)
            row.is_total = True
            return row

        def compute_row_header(row_config):
            header_row = self.get_empty_row_asdict(self.get_fields(), ReportValue(''))
            for k, fun in row_config.items():
                if hasattr(fun, 'caption'):
                    value = force_unicode(fun.caption)
                else:
                    value = '&nbsp;'
                header_row[k] = value
            row = self.reorder_dictrow(header_row)
            row = ReportRow(row)
            row.is_caption = True
            return row

        def group_m2m_field_values(gqs_values):
            values_results = []
            m2m_indexes = [index for ffield, lkfield, index, field in self.model_m2m_fields]

            def get_key_values(gqs_vals):
                return [v if index not in m2m_indexes else None for index, v in enumerate(gqs_vals)]

            # gqs_values needs to already be sorted on the same key function
            # for groupby to work properly
            gqs_values = sorted(gqs_values, key=get_key_values)
            res = groupby(gqs_values, key=get_key_values)
            row_values = {}
            for key, values in res:
                row_values = dict([(index, []) for index in m2m_indexes])
                for v in values:
                    for index in m2m_indexes:
                        if v[index] not in row_values[index]:
                            row_values[index].append(v[index])
                for index, vals in row_values.items():
                    key[index] = vals
                values_results.append(key)
            return values_results

        qs_list = get_with_dotvalues(qs_list)
        if self.model_m2m_fields:
            qs_list = group_m2m_field_values(qs_list)

        if groupby_data and groupby_data['groupby']:
            groupby_field = groupby_data['groupby']
            if groupby_field in self.override_group_value:
                transform_fn = self.override_group_value.get(groupby_field)
                g = groupby(qs_list, lambda x: transform_fn(x[ffields.index(groupby_field)]))
            else:
                g = groupby(qs_list, lambda x: x[ffields.index(groupby_field)])
        else:
            g = groupby(qs_list, lambda x: None)

        row_report_totals = self.get_empty_row_asdict(self.report_totals, [])
        for grouper, resources in g:
            rows = list()
            row_group_totals = self.get_empty_row_asdict(self.group_totals, [])
            for resource in resources:
                row = ReportRow()
                if isinstance(resource, (tuple, list)):
                    for index, value in enumerate(resource):
                        if ffields[index] in self.group_totals:
                            row_group_totals[ffields[index]].append(value)
                        elif ffields[index] in self.report_totals:
                            row_report_totals[ffields[index]].append(value)
                        value = self._get_value_text(index, value)
                        value = ReportValue(value)
                        if ffields[index] in self.override_field_values:
                            value.to_value = self.override_field_values[ffields[index]]
                        if ffields[index] in self.override_field_formats:
                            value.format = self.override_field_formats[ffields[index]]
                        row.append(value)
                else:
                    for index, column in enumerate(ffields):
                        value = get_field_value(resource, column)
                        if ffields[index] in self.group_totals:
                            row_group_totals[ffields[index]].append(value)
                        elif ffields[index] in self.report_totals:
                            row_report_totals[ffields[index]].append(value)
                        value = self._get_value_text(index, value)
                        value = ReportValue(value)
                        if column in self.override_field_values:
                            value.to_value = self.override_field_values[column]
                        if column in self.override_field_formats:
                            value.format = self.override_field_formats[column]
                        row.append(value)
                rows.append(row)
            if row_group_totals:
                if groupby_data['groupby']:
                    header_group_total = compute_row_header(self.group_totals)
                    row = compute_row_totals(self.group_totals, row_group_totals, is_group_total=True)
                    rows.append(header_group_total)
                    rows.append(row)
                for k, v in row_group_totals.items():
                    if k in row_report_totals:
                        row_report_totals[k].extend(v)

            if groupby_data and groupby_data['groupby']:
                grouper = self._get_grouper_text(groupby_data['groupby'], grouper)
            else:
                grouper = None
            if isinstance(grouper, (list, tuple)):
                grouper = grouper[0]
            report_rows.append([grouper, rows])
        if self.has_report_totals():
            header_report_total = compute_row_header(self.report_totals)
            row = compute_row_totals(self.report_totals, row_report_totals, is_report_total=True)
            header_report_total.is_report_totals = True
            row.is_report_totals = True
            report_rows.append([_('Totals'), [header_report_total, row]])

        return report_rows



# -*- coding: utf-8 -*-
import cStringIO as StringIO
import ho.pisa as pisa
from django.template.loader import get_template
from django.template import Context
from django.http import HttpResponse
from cgi import escape


def render_to_pdf(report, template_src, context_dict, pdf_encoding='UTF-8'):
    template = get_template(template_src)
    context = Context(context_dict)
    html = template.render(context)
    result = StringIO.StringIO()

    pdf = pisa.pisaDocument(StringIO.StringIO(html.encode(pdf_encoding)), result)
    if not pdf.err:
        response = HttpResponse(result.getvalue(), mimetype='application/pdf')
        response['Content-Disposition'] = 'attachment; filename=%s.pdf' % report.slug
        return response
    return HttpResponse('We had some errors<pre>%s</pre>' % escape(html))



# -*- coding: utf-8 -*-
from os.path import join, dirname
from setuptools import setup, find_packages


version = __import__('model_report').__version__


LONG_DESCRIPTION = """
django-model-report
===================

django reports integrated with highcharts

    $ git clone git://github.com/juanpex/django-model-report.git
"""


def long_description():
    try:
        return open(join(dirname(__file__), 'README.md')).read()
    except IOError:
        return LONG_DESCRIPTION


setup(name='django-model-report',
      version=version,
      author='juanpex',
      author_email='jpma55@gmail.com',
      description='Django reports integrated with highcharts.',
      license='BSD',
      keywords='django, model, report, reports, highcharts, chart, charts',
      url='https://github.com/juanpex/django-model-report',
      packages=find_packages(),
      package_data={'model_report': ['locale/*/LC_MESSAGES/*']},
      long_description=long_description(),
      install_requires=['django>=1.2.5', ],
      classifiers=['Framework :: Django',
                   'Development Status :: 3 - Alpha',
                   'Topic :: Internet',
                   'License :: OSI Approved :: BSD License',
                   'Intended Audience :: Developers',
                   'Environment :: Web Environment',
                   'Programming Language :: Python :: 2.5',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7'])



# -*- coding: utf-8 -*-
from model_report.highcharts.base import true, false, null, Solid, outside, undefined, _, CollectionObject, DictObject


def get_highchart_data():

    ChartData = {
        'alignTicks': true,
        'animation': true,
        'backgroundColor': "#FFFFFF",
        'borderColor': "#4572A7",
        'borderRadius': 5,
        'borderWidth': 0,
        'className': "",
        'defaultSeriesType': null,
        'events': {},
        'height': null,
        'ignoreHiddenSeries': true,
        'inverted': false,
        'margin': [null],
        'marginTop': null,
        'marginRight': 0,
        'marginBottom': 0,
        'marginLeft': 80,
        'plotBackgroundColor': null,
        'plotBackgroundImage': null,
        'plotBorderColor': "#C0C0C0",
        'plotBorderWidth': 0,
        'plotShadow': false,
        'reflow': true,
        'renderTo': null,
        'resetZoomButton': {},
        # 'selectionMarkerFill': rgba(69, 114, 167, 0.25),
        'shadow': false,
        'showAxes': false,
        'spacingTop': 10,
        'spacingRight': 10,
        'spacingBottom': 15,
        'spacingLeft': 10,
        'style': {},
        'type': "line",
        'width': null,
        'zoomType': ""
    }

    CreditsData = {
        'enabled': false
    }

    TitleData = {
        'align': "center",
        'floating': false,
        'margin': 15,
        'text': null,
        'style': {},
        'verticalAlign': "top",
        'x': 15,
        'y': 15
    }

    SubtitleData = {
        'align': "center",
        'floating': false,
        'text': "",
        'style': {},
        'verticalAlign': "top",
        'x': 0,
        'y': 30
    }

    #Solid = 'Solid'
    #outside = 'outside'
    #undefined = 'undefined'

    xAxisData = {
        'allowDecimals': true,
        'alternateGridColor': null,
        'categories': [],
        'dateTimeLabelFormats': null,
        'endOnTick': false,
        'events': {},
        'gridLineColor': "#C0C0C0",
        'gridLineDashStyle': Solid,
        'gridLineWidth': 0,
        'id': null,
        'labels': {},
        'lineColor': "#C0D0E0",
        'lineWidth': 1,
        'linkedTo': null,
        'max': null,
        'maxPadding': 0.01,
        'maxZoom': null,
        'min': null,
        'minorGridLineColor': '#E0E0E0',
        'minorGridLineDashStyle': Solid,
        'minorGridLineWidth': 1,
        'minorTickColor': '#A0A0A0',
        'minorTickInterval': null,
        'minorTickLength': 2,
        'minorTickPosition': outside,
        'minorTickWidth': 0,
        'minPadding': 0.01,
        'minRange': null,
        'offset': 0,
        'opposite': false,
        'plotBands': [{}],
        'plotLines': [{}],
        'reversed': false,
        'showFirstLabel': true,
        'showLastLabel': false,
        'startOfWeek': 1,
        'startOnTick': false,
        'tickColor': '#C0D0E0',
        'tickInterval': null,
        'tickLength': 5,
        'tickmarkPlacement': "between",
        'tickPixelInterval': null,
        'tickPosition': "outside",
        'tickWidth': 1,
        'title': DictObject(**TitleData),
        'type': "linear"
    }

    yAxisPlotData = {
        'value': null,
        'width': null,
        'color': null
    }

    yAxisData = {
        'endOnTick': true,
        'gridLineWidth': 1,
        'lineWidth': 0,
        'maxPadding': 0.05,
        'minPadding': 0.05,
        'showLastLabel': true,
        'stackLabels': {},
        'startOnTick': true,
        'tickWidth': 0,
        'allowDecimals': true,
        'alternateGridColor': null,
        'categories': [],
        'dateTimeLabelFormats': null,
        'events': {},
        'gridLineColor': "#C0C0C0",
        'gridLineDashStyle': Solid,
        'id': null,
        'labels': {},
        'lineColor': "#C0D0E0",
        'linkedTo': null,
        'max': null,
        'maxZoom': null,
        'min': null,
        'minorGridLineColor': '#E0E0E0',
        'minorGridLineDashStyle': Solid,
        'minorGridLineWidth': 1,
        'minorTickColor': '#A0A0A0',
        'minorTickInterval': null,
        'minorTickLength': 2,
        'minorTickPosition': outside,
        'minorTickWidth': 0,
        'minRange': null,
        'offset': 0,
        'opposite': false,
        'plotBands': [{}],
        'plotLines': CollectionObject(),
        'reversed': false,
        'showFirstLabel': true,
        'startOfWeek': 1,
        'tickColor': '#C0D0E0',
        'tickInterval': null,
        'tickLength': 5,
        'tickmarkPlacement': "between",
        'tickPixelInterval': null,
        'tickPosition': "outside",
        'title': DictObject(**TitleData),
        'type': "linear"
    }

    TooltipData = {
        'backgroundColor': "rgba(255, 255, 255, .85)",
        'borderColor': "auto",
        'borderRadius': 5,
        'borderWidth': 2,
        'crosshairs': null,
        'enabled': true,
        'footerFormat': false,
        'formatter': "function() { return '<b>'+ this.series.name + '</b><br/>' + this.x + ': ' + this.y +'C'; }",
        'pointFormat': null,
        'positioner': null,
        'shadow': true,
        'shared': false,
        'snap': 10 / 25,
        'style': {},
        'valueDecimals': null,
        'valuePrefix': "",
        'valueSuffix': "",
        'xDateFormat': null,
        'useHTML': false
    }

    SeriesData = {
        'data': [],
        'dataParser': null,
        'dataURL': null,
        'legendIndex': undefined,
        'name': '',
        'stack': null,
        'type': "line",
        'xAxis': 0,
        'yAxis': 0
    }

    SeriesCollection = CollectionObject

    plotOptionsDataPIE_Labels = {
        'align': null,
        'connectorWidth': 1,
        'connectorColor': "{point.color}",
        'connectorPadding': 5,
        'distance': 30,
        'enabled': true,
        'softConnector': true,
        'backgroundColor': undefined,
        'borderColor': undefined,
        'borderRadius': 0,
        'borderWidth': 0,
        'color': null,
        'formatter': "",
        'padding': 2,
        'rotation': 0,
        'shadow': false,
        'staggerLines': "",
        'step': "",
        'style': {},
        'x': 0,
        'y': -6,
        'overflow': undefined
    }

    plotOptionsDataPIE = {
        'borderColor': "#FFFFFF",
        'borderWidth': 1,
        'center': ['50%', '50%'],
        'innerSize': 0,
        'showInLegend': false,
        'size': "75%",
        'slicedOffset': 10,
        'allowPointSelect': false,
        'animation': true,
        'color': "#000000",
        'connectNulls': false,
        'cropThreshold': 300,
        'cursor': '',
        'dashStyle': null,
        'dataLabels': DictObject(**plotOptionsDataPIE_Labels),
        'enableMouseTracking': true,
        'events': {},
        'id': null,
        'lineWidth': 2,
        'marker': {},
        'point': {},
        'pointStart': 0,
        'pointInterval': 1,
        'selected': false,
        'shadow': true,
        'showCheckbox': false,
        'stacking': null,
        'states': {},
        'stickyTracking': true,
        'tooltip': {},
        'turboThreshold': 1000,
        'visible': true,
        'zIndex': null
    }

    plotOptionsDataCOLUMN = {
        'borderColor': '#FFFFFF',
        'borderRadius': 0,
        'borderWidth': 1,
        'colorByPoint': false,
        'cropThreshold': 50,
        'groupPadding': 0.2,
        'minPointLength': 0,
        'pointPadding': 0.1,
        'pointWidth': null,
        'allowPointSelect': false,
        'animation': true,
        # 'color': ,
        'connectNulls': false,
        'cursor': '',
        'dashStyle': null,
        'dataLabels': {},
        'enableMouseTracking': true,
        'events': {},
        'id': null,
        'lineWidth': 2,
        'marker': {},
        'point': {},
        'pointStart': 0,
        'pointInterval': 1,
        'selected': false,
        'shadow': true,
        'showCheckbox': false,
        'showInLegend': true,
        'stacking': null,
        'states': {},
        'stickyTracking': true,
        'tooltip': {},
        'turboThreshold': 1000,
        'visible': true,
        'zIndex': null
    }

    plotOptionsDataLINEdataLabels = {
        'align': "center",
        'backgroundColor': undefined,
        'borderColor': undefined,
        'borderRadius': 0,
        'borderWidth': 0,
        'color': null,
        'crop': true,
        'enabled': false,
        #'formatter':,
        'overflow': undefined,
        'padding': 2,
        'rotation': 0,
        'shadow': false,
        #'staggerLines':,
        #'step':,
        #'style':,
        'useHTML': false,
        # 'verticalAlign':,
        'x': 0,
        'y': -6
    }

    plotOptionsDataLINE = {
        'allowPointSelect': false,
        'animation': true,
        # 'color': ,
        'connectEnds': true,
        'connectNulls': false,
        'cropThreshold': 300,
        'cursor': '',
        'dashStyle': null,
        'dataLabels': DictObject(**plotOptionsDataLINEdataLabels),
        'enableMouseTracking': true,
        'events': {},
        'id': null,
        'lineWidth': 2,
        'marker': {},
        'point': {},
        'pointInterval': 1,
        # 'pointPlacement':,
        'pointStart': 0,
        'selected': false,
        'shadow': true,
        'showCheckbox': false,
        'showInLegend': true,
        'stacking': null,
        'states': {},
        'step': false,
        'stickyTracking': true,
        'tooltip': {},
        'turboThreshold': 1000,
        'visible': true,
        'zIndex': null,
    }

    plotOptionsData = {
        'area': {},
        'areaspline': {},
        'bar': {},
        'column': DictObject(**plotOptionsDataCOLUMN),
        'line': DictObject(**plotOptionsDataLINE),
        'pie': DictObject(**plotOptionsDataPIE),
        'series': [{}],
        'scatter': {},
        'spline': {}
    }

    legendData = {
        'align': "center",
        'backgroundColor': null,
        'borderColor': '#909090',
        'borderRadius': 5,
        'borderWidth': 1,
        'enabled': true,
        'floating': false,
        'itemHiddenStyle': {},
        'itemHoverStyle': {},
        'itemMarginBottom': 0,
        'itemMarginTop': 0,
        'itemStyle': {},
        'itemWidth': null,
        'layout': "horizontal",
        'labelFormatter': null,
        'lineHeight': 16,
        'margin': 15,
        'navigation': {},
        'padding': 8,
        'reversed': false,
        'rtl': false,
        'shadow': false,
        'style': {},
        'symbolPadding': 5,
        'symbolWidth': 30,
        'verticalAlign': "bottom",
        'width': null,
        'x': 0,
        'y': 0
    }

    langData = {
        'decimalPoint': ".",
        'downloadPNG': _("Download PNG image"),
        'downloadJPEG': _("Download JPEG image"),
        'downloadPDF': _("Download PDF document"),
        'downloadSVG': _("Download SVG vector image"),
        'exportButtonTitle': _("Export to raster or vector image"),
        'loading': _('Loading...'),
        'months': [_('January'), _('February'), _('March'), _('April'), _('May'), _('June'), _('July'), _('August'), _('September'), _('October'), _('November'), _('December')],
        'shortMonths': [_('Jan'), _('Feb'), _('Mar'), _('Apr'), _('May'), _('Jun'), _('Jul'), _('Aug'), _('Sep'), _('Oct'), _('Nov'), _('Dec')],
        'printButtonTitle': _("Print the chart"),
        'resetZoom': _('Reset zoom'),
        'resetZoomTitle': _('Reset zoom level 1:1'),
        'thousandsSep': ",",
        'weekdays': [_('Sunday'), _('Monday'), _('Tuesday'), _('Wednesday'), _('Thursday'), _('Friday'), _('Saturday')]
    }

    HighchartData = dict(
        credits=DictObject(**CreditsData),
        chart=DictObject(**ChartData),
        title=DictObject(**TitleData),
        subtitle=DictObject(**SubtitleData),
        xAxis=DictObject(**xAxisData),
        yAxis=DictObject(**yAxisData),
        tooltip=DictObject(**TooltipData),
        series=SeriesCollection(),
        plotOptions=DictObject(**plotOptionsData),
        legend=DictObject(**legendData),
        lang=DictObject(**langData),
        serie_obj=DictObject(**SeriesData),
        yAxisplot_obj=DictObject(**yAxisPlotData),
    )

    return HighchartData
